
/*                DTN's Super SmartCar NRF24 Radio Slave Controller Program
         Available at http://devestechnet.com/NewTech/SuperSmartCar#ArduinoSoftware
   The NRF 24 must be wired as follows: NRF 24 Slave [Onboard Car Controller] Using Arduino Mega
   2560: The CSN Pin must go to Pin 53 for the Mega. Connect: (L to R)
                                    GND - GND    BLK
                                    VCC - +3V3   WHI
                                    CE -  Pin 8  GRY
                                    CSN - Pin 53 PUR
                                    SCK - Pin 52 BLU
                                    MSI - Pin 51 GRN
                                    MSO - Pin 50 YEL
  This is the main program for the SuperSmartCar project. It is heavily dependent on the wiring
  scheme located here: http://devestechnet.com/NewTech/SuperSmartCar For best results, run all
  test programs on the website to ensure each individual system is wired and working correctly.

  Special Thanks to all that helped on the Arduino.cc forum. It's nice to know people who are
  helpful and generous with their knowledge and time. */

#include <SPI.h>               /* Serial Peripheral Interface Library (required for NRF Radio) */
#include <RF24.h>                                            /* NRF24 Radio Library (required) */
const byte initCEPin = 8;                                   /* Radio Chip Enable Digital Pin 8 */
const byte initCSNPin = 53;                            /* Radio SPI Chip Select Digital Pin 53 */
const uint64_t pipe = 0xE8E8F0F0E1LL;                             /* Radio's addressing system */
RF24 radio(initCEPin, initCSNPin);                                     /* Initialize the Radio */
int smartStream[9];                            /* Create Communication Array for all functions */

/*Right Rear Motor Number One*/
const byte enableA_RR = 44;                   /* Must be PWM (pins 2 through 13,44-46 on Mega) */
const byte in1 = 38;                                 /* Each IN pin sets up the Digital Matrix */
const byte in2 = 39;                                 /* for forward and reverse motor rotation */
float motorASpeed_RR = 0.0;                                  /* Start each motor at Zero Volts */

/* Left Rear Motor Number Two */
const byte enableB_LR = 45;                   /* Must be PWM (pins 2 through 13,44-46 on Mega) */
const byte in3 = 40;                                 /* Each IN pin sets up the Digital Matrix */
const byte in4 = 41;                                 /* for forward and reverse motor rotation */
float motorBSpeed_LR = 0.0;                                  /* Start each motor at Zero Volts */

/* Right Front Motor Number Three */
const byte enableC_RF = 46;                   /* Must be PWM (pins 2 through 13,44-46 on Mega) */
const byte in5 = 47;                                 /* Each IN pin sets up the Digital Matrix */
const byte in6 = 48;                                 /* for forward and reverse motor rotation */
float motorCSpeed_RF = 0.0;                                  /* Start each motor at Zero Volts */

/* Left Front Motor Number Four */
const byte enableD_LF = 2;                    /* Must be PWM (pins 2 through 13,44-46 on Mega) */
const byte in7 = 34;                                 /* Each IN pin sets up the Digital Matrix */
const byte in8 = 35;                                 /* for forward and reverse motor rotation */
float motorDSpeed_LF = 0.0;                                  /* Start each motor at Zero Volts */

float motorASpeedMax = 255.0;   /* Set the maximum motor speed for Right Rear Motor (A) to 255 */
float motorBSpeedMax = 255.0;    /* Set the maximum motor speed for Left Rear Motor (B) to 255 */
float motorCSpeedMax = 255.0;  /* Set the maximum motor speed for Right Front Motor (C) to 255 */
float motorDSpeedMax = 255.0;   /* Set the maximum motor speed for Left Front Motor (D) to 255 */

/* To find the median speed setting:      255 - 40 = 215 / 2 = 107.5 then 255 - 107.5 = 147.5  */
float motorASpeedMed = 147.5;  /* Set the median motor speed for Right Rear Motor (A) to 147.5 */
float motorBSpeedMed = 147.5;  /* Set the median motor speed for Right Rear Motor (B) to 147.5 */
float motorCSpeedMed = 147.5;  /* Set the median motor speed for Right Rear Motor (C) to 147.5 */
float motorDSpeedMed = 147.5;  /* Set the median motor speed for Right Rear Motor (D) to 147.5 */

float motorASpeedMin = 40.0;     /* Set to lowest value that makes Right Rear Motor (A) rotate */
float motorBSpeedMin = 40.0;      /* Set to lowest value that makes Left Rear Motor (B) rotate */
float motorCSpeedMin = 40.0;    /* Set to lowest value that makes Right Front Motor (C) rotate */
float motorDSpeedMin = 40.0;     /* Set to lowest value that makes Left Front Motor (D) rotate */

float motorASpeedOff = 0.0;      /* Set to lowest value that makes Right Rear Motor (A) rotate */
float motorBSpeedOff = 0.0;       /* Set to lowest value that makes Left Rear Motor (B) rotate */
float motorCSpeedOff = 0.0;     /* Set to lowest value that makes Right Front Motor (C) rotate */
float motorDSpeedOff = 0.0;      /* Set to lowest value that makes Left Front Motor (D) rotate */

const byte ledPin = 9;                           /* the Digital PWM pin the LED is attached to */
int ledState = LOW;                                           /* Start with the LED turned off */
unsigned long timeBefore = 0;           /* Time since the last reset or reload (used for delay)*/
const long millisDelay = 30;                                /* LED Blink Interval for liteItUp */

int joyXValue = 0;                            /* Begin with 0 power applied to Joystick X Axis */
int joyYValue = 0;                            /* Begin with 0 power applied to Joystick Y Axis */
int joyXValueMin = 0;                                   /* Set lowest power to 0 to Joystick X */
int joyYValueMin = 0;                                   /* Set lowest power to 0 to Joystick Y */
int joyXValueMid = 512;                             /* Set midrange power to 512 to Joystick X */
int joyYValueMid = 512;                             /* Set midrange power to 512 to Joystick Y */
int joyXValueMax = 1023;                           /* Set maxiumum power to 1023 to Joystick X */
int joyYValueMax = 1023;                            /* Set maximum power to 1023 to Joystick Y */

/*The reason for below is to overcome the Thumbsticks dead zone in the center of its movement*/
int joyXValueMidUpper = joyXValueMid + 10;   /* Set center dead range (high) of Joystick X +20 */
int joyYValueMidUpper = joyYValueMid + 10;   /* Set center dead range (high) of Joystick Y +20 */
int joyXValueMidLower = joyXValueMid - 10;    /* Set center dead range (low) of Joystick X -20 */
int joyYValueMidLower = joyYValueMid - 10;    /* Set center dead range (low) of Joystick Y -20 */

/* Sharp Front Facing IR Proximity Sensor Variables */
const byte faciaCenterSensorPin = A5;
float cmDistance;

/* FC-51 Rear Obstacle Detection IR Sensors (x3) */
const byte rearRightSensor = 30;                    /* Sensor output to Arduino Digital Pin 30 */
const byte rearMiddleSensor = 31;                   /* Sensor output to Arduino Digital Pin 31 */
const byte rearLeftSensor = 32;                     /* Sensor output to Arduino Digital Pin 32 */
int rearLeftDetect = HIGH;
int rearMidDetect = HIGH;                                            /* HIGH means NO OBSTACLE */
int rearRightDetect = HIGH;

/* Dual Servo Initialization */
#include <PWMServo.h>                                              /* Servo Library (required) */
PWMServo ServoPan;
PWMServo ServoTilt;
#define SERVO_PIN_A 11      /* These two pins for the servos are set in the PWMServo.h Library */
#define SERVO_PIN_B 12                 /* They are repeated here so you can see the pins used. */
const int servoDelay = 25;
int panLowLimit = 0;                       /* Set this variable for your Pan Servo's Low Limit */
int panHighLimit = 175;                   /* Set this variable for your Pan Servo's High Limit */
int tiltLowLimit = 0;                     /* Set this variable for your Tilt Servo's Low Limit */
int tiltHighLimit = 176;                 /* Set this variable for your Tilt Servo's High Limit */
int posPan = 90;                       /* Set the Pan Servo's Position to Center for stairScan */

/* HC-SR04 Sonar Sensors (x3) */
const byte echoPinL = 18;               /* Echo pins all use interrupts. This means we need to */
const byte trigPinL = 19;
const byte echoPinG = 3;              /* use the 6 pins that are designated Interrupt pins. On */
const byte trigPinG = 4;
const byte echoPinR = 20;        /* the Mega 2560 these are Pins 2(interrupt 0),3(interrupt 1) */
const byte trigPinR = 21;   /* 18(interrupt 5),19(interrupt 4),20(interrupt 3),21(interrupt 2) */
bool trigEndG = false;                                               /* trigger pulse finished */
bool trigEndL = false;
bool trigEndR = false;
volatile unsigned long retDurG;                                       /* return pulse duration */
volatile unsigned long retDurL;
volatile unsigned long retDurR;
volatile bool retCapG = false;                                        /* return pulse captured */
volatile bool retCapL = false;
volatile bool retCapR = false;
volatile double totalG;                                                    /* total round trip */
volatile double totalL;
volatile double totalR;

/* Controller Switch Assignments */
/* Thumbstick Switch Variables */
bool joySwitch = true;                                     /* Variable for the Joystick Switch */
const byte joySwStatePin = 26;  /* Digital Pin used to make state changes for toggle switching */
bool joySwPinState = true;
bool joySwitchPreVal = false;
/* Button Variables */
bool btnAValue = true;                                                           /* unAssigned */
bool btnBValue = true;                                   /* Assigned to Zero Turn to the Right */
bool btnCValue = true;                                                           /* unAssigned */
bool btnDValue = true;                                    /* Assigned to Zero Turn to the Left */
/* E Button Variables */
bool btnEValue = true;                                           /* Assigned to StairScan Mode */
const byte btnEStatePin = 29;   /* Digital Pin used to make state changes for toggle switching */
bool btnEPinState = false;
bool btnEPreVal = false;
long time = 0;                               /* Starts the timer at Zero for debounce purposes */
long debounce = 200;                                               /* Debounce in milliseconds */
/* F Button Variables */
bool btnFValue = true; /* Used for Test Mode so we can see communications between Master/Slave */
bool testMode = true;  /* Used for turning Test Mode on/off so we can use F for something else */

/* FC-03 Down-facing Line Tracking IR Sensors (x3) */
const byte sensorPinL = A1;                                    /* Left sensor to Analog pin A1 */
const byte sensorPinM = A2;                                  /* Middle sensor to Analog pin A2 */
const byte sensorPinR = A3;                                   /* Right sensor to Analog pin A3 */
int lSenseVal;               /* These are the actual values coming out of the sensors. You can */
int mSenseVal;                   /* use these numbers to manipulate your line tracking for more*/
int rSenseVal;                 /* accurate results. These numbers are not software adjustable. */
int lSenseLow = 75; int lSenseMid = 125; int lSenseHigh = 300;/*Sensors are slightly different */
int mSenseLow = 75; int mSenseMid = 125; int mSenseHigh = 300;  /* in output. Adjust these for */
int rSenseLow = 75; int rSenseMid = 125; int rSenseHigh = 300;        /* more accurate results */

void setup() {
  Serial.begin(115200);         /* Open Serial Comm for testing and smartStreamTest() Button F */
  pinMode(ledPin, OUTPUT);
  pinMode (enableA_RR, OUTPUT);        /* Both enable A and B are from the first (rear) module */
  pinMode (in1, OUTPUT);                   /* in1 through in4 are from the first (rear) module */
  pinMode (in2, OUTPUT);
  pinMode (enableB_LR, OUTPUT);
  pinMode (in3, OUTPUT);
  pinMode (in4, OUTPUT);
  pinMode (enableC_RF, OUTPUT);      /* Both enable C and D are from the second (front) module */
  pinMode (in5, OUTPUT);                 /* in5 through in8 are from the second (front) module */
  pinMode (in6, OUTPUT);
  pinMode (enableD_LF, OUTPUT);
  pinMode (in7, OUTPUT);
  pinMode (in8, OUTPUT);
  pinMode(rearRightSensor, INPUT);
  pinMode(rearMiddleSensor, INPUT);           /* FC-51 Rear Obstacle Detection IR Sensors (x3) */
  pinMode(rearLeftSensor, INPUT);
  pinMode(trigPinR, OUTPUT);
  pinMode(echoPinR, INPUT);
  pinMode(trigPinG, OUTPUT);                                     /* HC-SR04 Sonar Sensors (x3) */
  pinMode(echoPinG, INPUT);
  pinMode(trigPinL, OUTPUT);
  pinMode(echoPinL, INPUT);
  pinMode(joySwStatePin, OUTPUT);    /* Pin 26 used for digital State of the Thumbstick Switch */
  pinMode(btnEStatePin, OUTPUT);          /* Pin 29 used for digital State change for Button E */
  attachInterrupt(digitalPinToInterrupt(echoPinG), EchoPinISR_G, CHANGE);
  attachInterrupt(digitalPinToInterrupt(echoPinL), EchoPinISR_L, CHANGE);
  attachInterrupt(digitalPinToInterrupt(echoPinR), EchoPinISR_R, CHANGE);
  ServoPan.attach(SERVO_PIN_A); ServoPan.write((panHighLimit - panLowLimit) / 2);
  ServoTilt.attach(SERVO_PIN_B); ServoTilt.write((tiltHighLimit - tiltLowLimit) / 2);
  radio.begin();                               /* Open the NRF24 Transceiver for communucation */
  radio.openReadingPipe(1, pipe);                       /* Open two-way Comm to the Master Uno */
  radio.startListening();                                     /* Begin handshake communication */
}

void loop() {
  if (radio.available()) {                                        /* Detect Signal from Master */
    radioControl();
    if (joySwPinState == true) {                                          /* Line Tracker Mode */
      lineTracker();
    }
    else if (joySwPinState == false) {
      if (joyXValue > joyXValueMidUpper) {
        moveForward();
      }
      else if (joyXValue < joyXValueMidLower) {
        moveBackward();
      }
      else if (joyXValue > joyXValueMidLower && joyXValue < joyXValueMidUpper) {
        allStop();                                              /* if thumbstick is in neutral */
      }
      zeroTurn();                                /* Zero Turning Radius Left and Right Control */
      fc51RearSensors();
      sharpSensor();
      sonarSensors();
      stairScan();
    }
  }
  /* Test without Radio Here */
}

void radioControl() {
  radio.read(smartStream, sizeof(smartStream));               /* Read in the smartStream Array */
  smartStreamTest();                   /* Tests the Radio Stream when pressing (and holding F) */
  joyYValue = smartStream[0];                      /* Assigns array index 0 to joystick Y axis */
  joyXValue = smartStream[1];                      /* Assigns array index 1 to joystick X axis */
  joySwitch = smartStream[2];                      /* Assigns array index 2 to joystick switch */
  btnAValue = smartStream[3];                             /* Assigns array index 3 to button A */
  btnBValue = smartStream[4];                             /* Assigns array index 4 to button B */
  btnCValue = smartStream[5];                             /* Assigns array index 5 to button C */
  btnDValue = smartStream[6];                             /* Assigns array index 6 to button D */
  btnEValue = smartStream[7];                             /* Assigns array index 7 to button E */
  btnFValue = smartStream[8];                             /* Assigns array index 8 to button F */
  liteItUp();          /* LED Visual used to determine if the Radios are talking to each other */

  /*   The following code to the end of radioControl() and the associated global variables are */
  /* for changing the state of a pushbutton to emulate a toggle switch. This functionality is  */
  /* needed for changing to different modes of operation. */

  /* Line Tracker Mode Code */
  if (joySwitch == true && joySwitchPreVal == false && millis() - time > debounce) {
    if (joySwPinState == false) {                                       /* Uses Digital Pin 26 */
      joySwPinState = true;                 /* to determine the digital output's State. 0 or 1 */
    }
    else {
      joySwPinState = false;
    }
    time = millis();
  }
  digitalWrite(joySwStatePin, joySwPinState);
  joySwitchPreVal = false;
  joySwitchPreVal = joySwitch;                                   /* The buttons previous value */

  /* StairCase Mode Code */
  if (btnEValue == true && btnEPreVal == false && millis() - time > debounce) {
    if (btnEPinState == false) {                                        /* Uses Digital Pin 29 */
      btnEPinState = true;                  /* to determine the digital output's State. 0 or 1 */
    }
    else {
      btnEPinState = false;
    }
    time = millis();
  }
  digitalWrite(btnEStatePin, btnEPinState);
  btnEPreVal = false;
  btnEPreVal = btnEValue;                                        /* The buttons previous value */
}

void smartStreamTest() {
  testMode = true;
  if (btnFValue == false && testMode == true) {
    Serial.print(" Y-Axis: "); Serial.print(joyYValue); Serial.print("\t");  /* Press and hold */
    Serial.print(" X-Axis: "); Serial.print(joyXValue); Serial.print("\t");  /* F on Master to */
    Serial.print(" JoySW: ");  Serial.print(joySwitch); Serial.print("\t");  /* observe values */
    Serial.print(" ButnA: ");  Serial.print(btnAValue); Serial.print("\t");  /* from all nine  */
    Serial.print(" ButnB: ");  Serial.print(btnBValue); Serial.print("\t");  /* functions that */
    Serial.print(" ButnC: ");  Serial.print(btnCValue); Serial.print("\t");  /* are available  */
    Serial.print(" ButnD: ");  Serial.print(btnDValue); Serial.print("\t");  /* through Serial */
    Serial.print(" ButnE: ");  Serial.print(btnEValue); Serial.print("\t");  /* Monitor.       */
    Serial.print(" ButnF: ");  Serial.print(btnFValue); Serial.print("\t");  /* Confirms Radio!*/
    Serial.println("");       /* You know because these variables are local to the slave only! */
    testMode = false;
  }
}

void liteItUp() {           /* this is how you use a delay WITHOUT delaying the entire program */
  unsigned long currentTime = millis();
  if (currentTime - timeBefore >= millisDelay) {
    timeBefore = currentTime;
    if (ledState == LOW) {
      ledState = HIGH;
    } else {
      ledState = LOW;
    }
    digitalWrite(ledPin, ledState);
  }
}

void zeroTurn() {                  /* used to test the Zero Turning Radius features of the Car */
  if (btnDValue == false) {                 /* Left Turning Zero Turn for D button to the left */
    zeroTurnL();
  }
  if (btnBValue == false) {               /* Right Turning Zero Turn for B button to the right */
    zeroTurnR();
  }
}

void allStop() {
  digitalWrite (in1, LOW);
  digitalWrite (in2, LOW);
  digitalWrite (in3, LOW);
  digitalWrite (in4, LOW);
  digitalWrite (in5, LOW);
  digitalWrite (in6, LOW);
  digitalWrite (in7, LOW);
  digitalWrite (in8, LOW);
  analogWrite (enableA_RR, motorASpeedOff);
  analogWrite (enableB_LR, motorBSpeedOff);
  analogWrite (enableC_RF, motorCSpeedOff);
  analogWrite (enableD_LF, motorDSpeedOff);
}

void moveForward() {              /* move forwards based on the analog mapping of the joystick */
  digitalWrite (in1, LOW);
  digitalWrite (in2, HIGH);
  digitalWrite (in3, LOW);
  digitalWrite (in4, HIGH);
  digitalWrite (in5, LOW);
  digitalWrite (in6, HIGH);
  digitalWrite (in7, LOW);
  digitalWrite (in8, HIGH);
  analogWrite (enableA_RR, motorASpeed_RR);
  analogWrite (enableB_LR, motorBSpeed_LR);
  analogWrite (enableC_RF, motorCSpeed_RF);
  analogWrite (enableD_LF, motorDSpeed_LF);
  motorASpeed_RR = map(joyXValue, joyXValueMidUpper, joyXValueMax, motorASpeedMin, motorASpeedMax);
  motorBSpeed_LR = map(joyXValue, joyXValueMidUpper, joyXValueMax, motorBSpeedMin, motorBSpeedMax);
  motorCSpeed_RF = map(joyXValue, joyXValueMidUpper, joyXValueMax, motorCSpeedMin, motorCSpeedMax);
  motorDSpeed_LF = map(joyXValue, joyXValueMidUpper, joyXValueMax, motorDSpeedMin, motorDSpeedMax);
}

void moveForwardMax() {                                            /* go forwards at max speed */
  digitalWrite (in1, LOW);
  digitalWrite (in2, HIGH);
  digitalWrite (in3, LOW);
  digitalWrite (in4, HIGH);
  digitalWrite (in5, LOW);
  digitalWrite (in6, HIGH);
  digitalWrite (in7, LOW);
  digitalWrite (in8, HIGH);
  analogWrite (enableA_RR, motorASpeedMax);
  analogWrite (enableB_LR, motorBSpeedMax);
  analogWrite (enableC_RF, motorCSpeedMax);
  analogWrite (enableD_LF, motorDSpeedMax);
}

void moveBackward() {            /* move backwards based on the analog mapping of the joystick */
  digitalWrite (in1, HIGH);
  digitalWrite (in2, LOW);
  digitalWrite (in3, HIGH);
  digitalWrite (in4, LOW);
  digitalWrite (in5, HIGH);
  digitalWrite (in6, LOW);
  digitalWrite (in7, HIGH);
  digitalWrite (in8, LOW);
  analogWrite (enableA_RR, motorASpeed_RR);
  analogWrite (enableB_LR, motorBSpeed_LR);
  analogWrite (enableC_RF, motorCSpeed_RF);
  analogWrite (enableD_LF, motorDSpeed_LF);
  motorASpeed_RR = map(joyXValue, joyXValueMidLower, joyXValueMin, motorASpeedMin, motorASpeedMax);
  motorBSpeed_LR = map(joyXValue, joyXValueMidLower, joyXValueMin, motorBSpeedMin, motorBSpeedMax);
  motorCSpeed_RF = map(joyXValue, joyXValueMidLower, joyXValueMin, motorCSpeedMin, motorCSpeedMax);
  motorDSpeed_LF = map(joyXValue, joyXValueMidLower, joyXValueMin, motorDSpeedMin, motorDSpeedMax);
}

void moveBackwardMax() {                                          /* go backwards at max speed */
  digitalWrite (in1, HIGH);
  digitalWrite (in2, LOW);
  digitalWrite (in3, HIGH);
  digitalWrite (in4, LOW);
  digitalWrite (in5, HIGH);
  digitalWrite (in6, LOW);
  digitalWrite (in7, HIGH);
  digitalWrite (in8, LOW);
  analogWrite (enableA_RR, motorASpeedMax);
  analogWrite (enableB_LR, motorBSpeedMax);
  analogWrite (enableC_RF, motorCSpeedMax);
  analogWrite (enableD_LF, motorDSpeedMax);
}

void zeroTurnL() {
  digitalWrite (in1, LOW);                           /* Right Rear Motor needs to run forwards */
  digitalWrite (in2, HIGH);
  digitalWrite (in3, HIGH);                          /* Left Rear Motor needs to run backwards */
  digitalWrite (in4, LOW);
  digitalWrite (in5, LOW);                          /* Right Front Motor needs to run forwards */
  digitalWrite (in6, HIGH);
  digitalWrite (in7, HIGH);                         /* Left Front Motor needs to run backwards */
  digitalWrite (in8, LOW);
  analogWrite (enableA_RR, motorASpeedMax);
  analogWrite (enableB_LR, motorBSpeedMax);
  analogWrite (enableC_RF, motorCSpeedMax);
  analogWrite (enableD_LF, motorDSpeedMax);
}

void zeroTurnR() {
  digitalWrite (in1, HIGH);                         /* Right Rear Motor needs to run backwards */
  digitalWrite (in2, LOW);
  digitalWrite (in3, LOW);                            /* Left Rear Motor needs to run forwards */
  digitalWrite (in4, HIGH);
  digitalWrite (in5, HIGH);                        /* Right Front Motor needs to run backwards */
  digitalWrite (in6, LOW);
  digitalWrite (in7, LOW);                           /* Left Front Motor needs to run forwards */
  digitalWrite (in8, HIGH);
  analogWrite (enableA_RR, motorASpeedMax);
  analogWrite (enableB_LR, motorBSpeedMax);
  analogWrite (enableC_RF, motorCSpeedMax);
  analogWrite (enableD_LF, motorDSpeedMax);
}

void fc51RearSensors() {                      /* FC-51 Rear Obstacle Detection IR Sensors (x3) */
  rearRightDetect = digitalRead(rearRightSensor);
  rearMidDetect = digitalRead(rearMiddleSensor);
  rearLeftDetect = digitalRead(rearLeftSensor);
  if (rearRightDetect == 0 || rearMidDetect == 0 || rearLeftDetect == 0) {
    moveForwardMax();
  }
}

void sharpSensor() {                                  /* Sharp 2Y0A21 IR Proximity Sensor Code */
  int rawSharpOutput = analogRead(faciaCenterSensorPin);
  if (rawSharpOutput >= 250) {
    moveBackwardMax();
  }
  else {
    joyXValue;
  }
}

void sonarSensors() {
  pingG();                                              /* HC-SR04 Gimbal Sensor Start Pinging */
  pingL();                                                /* HC-SR04 Left Sensor Start Pinging */
  pingR();                                               /* HC-SR04 Right Sensor Start Pinging */
  if (btnEPinState == false) {                      /* Toggle Button E to make stairScan false */
    if (totalG <= 12 || totalL <= 6 || totalR <= 6) {
      moveBackwardMax();
    }
    else {
      joyXValue;
    }
  }
  else if (btnEPinState == true) {                   /* Toggle Button E to make stairScan true */
    if (totalG >= 5 && totalG <= 20 || totalL <= 6 || totalR <= 6) {
      moveBackwardMax();
    }
    else {
      joyXValue;
    }
  }
}

void stairScan() {             /* Positions the Tilt Servo for scanning for stairs or dropoffs */
  if (btnEPinState == true) {                             /* Toggle Button E to make this true */
    int setPosAngle = 30;
    ServoTilt.write(setPosAngle);
    ServoPan.write(posPan);
  }
  else {
    ServoPan.write((panHighLimit - panLowLimit) / 2);
    ServoTilt.write((tiltHighLimit - tiltLowLimit) / 2);
  }
}

void pingG() {
  if (retCapG == true && trigEndG == true) {
    totalG = ((retDurG / 2) / 73.0);
    retCapG = false; //reset
    trigEndG = false;
  }
  const unsigned int pingSpeed = 90;
  static unsigned long pingTimerG = millis();
  if (millis() >= pingTimerG && trigEndG == false) {
    pingTimerG += pingSpeed;
    digitalWrite(trigPinG, HIGH);
    delayMicroseconds(10);
    digitalWrite(trigPinG, LOW);
    trigEndG = true;
  }
}

void EchoPinISR_G() {        /* this is the gimbal sensors interrupt code. Allows us to do two */
  if (retCapG == false) {                /* things at once using Arduinos Interrupt capability */
    static unsigned long startTime;        /* meaning.. Ping and Echo while doing other things */
    if (digitalRead(echoPinG)) // Gone HIGH
      startTime = micros();
    else { // Gone LOW
      retDurG = micros() - startTime;
      retCapG = true;
    }
  }
}

void pingL() {
  if (retCapL == true && trigEndL == true) {
    totalL = ((retDurL / 2) / 73.0);
    retCapL = false; //reset
    trigEndL = false;
  }
  const unsigned int pingSpeed = 90;
  static unsigned long pingTimerL = millis() + 30;
  if (millis() >= pingTimerL && trigEndL == false) {
    pingTimerL += pingSpeed;
    digitalWrite(trigPinL, HIGH);
    delayMicroseconds(10);
    digitalWrite(trigPinL, LOW);
    trigEndL = true;
  }
}

void EchoPinISR_L() {          /* this is the left sensors interrupt code. Allows us to do two */
  if (retCapL == false) {                /* things at once using Arduinos Interrupt capability */
    static unsigned long startTime;        /* meaning.. Ping and Echo while doing other things */
    if (digitalRead(echoPinL)) // Gone HIGH
      startTime = micros();
    else { // Gone LOW
      retDurL = micros() - startTime;
      retCapL = true;
    }
  }
}

void pingR() {
  if (retCapR == true && trigEndR == true) {
    totalR = ((retDurR / 2) / 73.0);
    retCapR = false; //reset
    trigEndR = false;
  }
  const unsigned int pingSpeed = 90;
  static unsigned long pingTimerR = millis() + 60;
  if (millis() >= pingTimerR && trigEndR == false) {
    pingTimerR += pingSpeed;
    digitalWrite(trigPinR, HIGH);
    delayMicroseconds(10);
    digitalWrite(trigPinR, LOW);
    trigEndR = true;
  }
}

void EchoPinISR_R() {         /* this is the right sensors interrupt code. Allows us to do two */
  if (retCapR == false) {                /* things at once using Arduinos Interrupt capability */
    static unsigned long startTime;        /* meaning.. Ping and Echo while doing other things */
    if (digitalRead(echoPinR)) // Gone HIGH
      startTime = micros();
    else { // Gone LOW
      retDurR = micros() - startTime;
      retCapR = true;
    }
  }
}

void lineTracker() {
  if (joySwPinState == true) {
    lSenseVal = analogRead (sensorPinL);
    mSenseVal = analogRead (sensorPinM);
    rSenseVal = analogRead (sensorPinR);
    Serial.print(" Left: ");
    Serial.print(lSenseVal);
    Serial.print("\t");
    Serial.print("Center: ");
    Serial.print(mSenseVal);
    Serial.print("\t");
    Serial.print("Right: ");
    Serial.print(rSenseVal);
    Serial.print("\t");
    Serial.println("");
    if ((lSenseVal > 10) && (mSenseVal > 120) && (rSenseVal < 120)) {
      Serial.println("                                             You are: On Course");
      digitalWrite (in1, LOW);
      digitalWrite (in2, HIGH);
      digitalWrite (in3, LOW);
      digitalWrite (in4, HIGH);
      digitalWrite (in5, LOW);
      digitalWrite (in6, HIGH);
      digitalWrite (in7, LOW);
      digitalWrite (in8, HIGH);
      analogWrite (enableA_RR, 140);
      analogWrite (enableB_LR, 140);
      analogWrite (enableC_RF, 140);
      analogWrite (enableD_LF, 140);

    }
    else if ((lSenseVal > 100) && (lSenseVal < 1023) && (rSenseVal < 100)) {
      Serial.println("                                             You are: Veering to the Right. CORRECT LEFT!");
      digitalWrite (in1, LOW);
      digitalWrite (in2, HIGH);
      digitalWrite (in3, LOW);
      digitalWrite (in4, LOW);
      digitalWrite (in5, LOW);
      digitalWrite (in6, HIGH);
      digitalWrite (in7, LOW);
      digitalWrite (in8, LOW);
      analogWrite (enableA_RR, 225);
      analogWrite (enableB_LR, 0);
      analogWrite (enableC_RF, 225);
      analogWrite (enableD_LF, 0);
    }
    else if ((rSenseVal > 120) && (rSenseVal < 1023) && (lSenseVal < 100)) {
      Serial.println("                                             You are: Veering to the Left. CORRECT RIGHT!");
      digitalWrite (in1, LOW);
      digitalWrite (in2, LOW);
      digitalWrite (in3, LOW);
      digitalWrite (in4, HIGH);
      digitalWrite (in5, LOW);
      digitalWrite (in6, LOW);
      digitalWrite (in7, LOW);
      digitalWrite (in8, HIGH);
      analogWrite (enableA_RR, 0);
      analogWrite (enableB_LR, 225);
      analogWrite (enableC_RF, 0);
      analogWrite (enableD_LF, 225);
    }
    if ((lSenseVal > 100 && rSenseVal > 100 && mSenseVal > 100) &&
        (lSenseVal > 100 && mSenseVal > 100 && rSenseVal > 100)) {
      Serial.println("                                                             You picked me up! ALL STOP!");
      allStop();
    }
    else if ((lSenseVal < 100) && (rSenseVal < 100) && (mSenseVal < 100)) {
      Serial.println("                                                             Not enough input! ALL STOP!");
      allStop();
    }
    delay(2);
  }
}
